// -*-c++-*-
/*
The MIT License (MIT)

Copyright (c) 2017 J. Eric Townsend

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/* in this script we do some basic interrupt examples

 * A1 potentiometer, controls on duration
 * A2 potentiometer, controls off duration
 *
 * output
 * D3 LED
 * D4 LED
 * D5 LED
 *
 */

static const int potOnPin = A1;
static const int potOffPin = A2;

static const int togglePin = 3;

static const int LED1Pin = 4;
static const int LED2Pin = 5;
static const int LED3Pin = 6;

int LEDOnDuration = 1000;  // milliseconds
int LEDOffDuration = 1000;  // milliseconds
long LEDOnTime = 0;
long LEDOffTime = 0;;
bool LEDState = false;
bool LEDStatePrev = false;

// variable for the ISR
// Q: should this be volatile?
// A: if any other part of the sketch is going to modifiy it, yes
//    but it's only modified in the ISR so we can leave it as a normal int
int toggleState = 0;

const bool isInterrupt = true;

// the interrupt method
// NOTE:  we shouldn't use Serial.prinln(), delay(), and many
// other functions in an interrupt method
void SwitchPressed()
{
    if (digitalRead(togglePin) == HIGH) {
        // this is why people hate C
        toggleState == 7 ? toggleState = 0 : toggleState++;
    }
}

void setup() {
    Serial.begin(115200);

    pinMode(potOnPin, INPUT);
    pinMode(potOffPin, INPUT);
    pinMode(togglePin, INPUT);
    
    pinMode(LED1Pin, OUTPUT);
    pinMode(LED2Pin, OUTPUT);
    pinMode(LED3Pin, OUTPUT);

    // attach the interrupt pin to a method
    if (isInterrupt) {
        attachInterrupt (digitalPinToInterrupt (togglePin), SwitchPressed, CHANGE);
    }
}

void loop() {


    // take input from the pots to determine on/off times
    int pot = analogRead(potOnPin);
    if (pot != LEDOnDuration) {
        LEDOnDuration = pot;
    }
    pot = analogRead(potOffPin);
    if (pot != LEDOffDuration) {
        LEDOffDuration = pot;
    }

    long now = millis();

    // set the LED state
    if (!LEDState && (now - LEDOffTime  >= LEDOffDuration)) {
        LEDState = true;
        LEDOnTime = now;
    }
    else if (LEDState && (now - LEDOnTime >= LEDOnDuration)) {
        LEDState = false;
        LEDOffTime = now;
    }

    if (!isInterrupt) {
        // this is how we would read a temporary switch without
        // interrupts.  The problem is that the loop is going so fast that
        // "one press" increments several times before I can release the
        // button.
        if (digitalRead(togglePin)) {
            // this is why people hate C
            toggleState == 7 ? toggleState = 0 : toggleState++;
            Serial.println(toggleState);
        }
    }

    if (LEDState != LEDStatePrev) {
        LEDStatePrev = LEDState;
        switch (LEDState) {
        case true:
            // Arduino has some clever ways to get around
            // K&R C syntax for reading bits
//https://www.arduino.cc/reference/en/language/functions/bits-and-bytes/bitread/
            if (bitRead(toggleState, 0)) {
                digitalWrite(LED1Pin, HIGH);
            }
            if (bitRead(toggleState, 1)) {
                digitalWrite(LED2Pin, HIGH);
            }
            if (bitRead(toggleState, 2)) {
                digitalWrite(LED3Pin, HIGH);
            }
            break;
        case false:
            digitalWrite(LED1Pin, LOW);
            digitalWrite(LED2Pin, LOW);
            digitalWrite(LED3Pin, LOW);
            break;
        }
    }
}

