// -*-c++-*-
/*
The MIT License (MIT)

Copyright (c) 2017 J. Eric Townsend

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/


/*

  TODO: try the different state changes that trigger an interrupt

 */


static const int potOnPin = A1;
static const int potOffPin = A2;


static const int togglePin = 3;

int LEDOnDuration = 1000;  // milliseconds
int LEDOffDuration = 1000;  // milliseconds
long LEDOnTime = 0;
long LEDOffTime = 0;;
bool LEDState = false;
bool LEDStatePrev = false;
int toggleState = 0;


void SwitchPressed()
{
    if (digitalRead(togglePin) == HIGH) {
        // this is why people hate C
        toggleState == 7 ? toggleState = 0 : toggleState++;
    }
}

void setup() {
    Serial.begin(115200);

    pinMode(potOnPin, INPUT);
    pinMode(potOffPin, INPUT);
    pinMode(togglePin, INPUT);
    
    // attach the interrupt pin to a method
/*
from <https://www.arduino.cc/reference/en/language/functions/external-interrupts/attachinterrupt/>

    LOW to trigger the interrupt whenever the pin is low,

    CHANGE to trigger the interrupt whenever the pin changes value

    RISING to trigger when the pin goes from low to high,

    FALLING for when the pin goes from high to low.
    The Due, Zero and MKR1000 boards allows also:

    HIGH to trigger the interrupt whenever the pin is high.

 */

    attachInterrupt (digitalPinToInterrupt (togglePin), SwitchPressed, FALLING);

}

void loop() {


    // take input from the pots to determine on/off times
    int pot = analogRead(potOnPin);
    if (pot != LEDOnDuration) {
        LEDOnDuration = pot;
    }
    pot = analogRead(potOffPin);
    if (pot != LEDOffDuration) {
        LEDOffDuration = pot;
    }

    long now = millis();

    // set the LED state
    if (!LEDState && (now - LEDOffTime  >= LEDOffDuration)) {
        LEDState = true;
        LEDOnTime = now;
    }
    else if (LEDState && (now - LEDOnTime >= LEDOnDuration)) {
        LEDState = false;
        LEDOffTime = now;
    }

    if (LEDState != LEDStatePrev) {
        LEDStatePrev = LEDState;
        switch (LEDState) {
        case true:
            // Arduino has some clever ways to get around
            // K&R C syntax for reading bits
//https://www.arduino.cc/reference/en/language/functions/bits-and-bytes/bitread/
            if (bitRead(toggleState, 2)) {
                Serial.print("1 ");
            }
            else  {
                Serial.print("0 ");
            }
            if (bitRead(toggleState, 1)) {
                Serial.print("1 ");
            }
            else  {
                Serial.print("0 ");
            }
            if (bitRead(toggleState, 0)) {
                Serial.println("1 ");
            }
            else  {
                Serial.println("0 ");
            }
            break;
        }
    }
    
}
